var
  CleanShortDesc,CleanLongDesc,WmVisitInstructions : String;
  BeforeCalled : Boolean;

function GetRunningScriptDir : String;
begin
  Result := ReplaceString(GEOGET_SCRIPTFULLNAME, GEOGET_SCRIPTNAME,'');
end;

{Metoda vrac cestu k adresi s ikonami}
function GetIconFolder : string;
begin
   Result := RelToAbsPath(ICON_FOLDER,GetRunningScriptDir);
end;

{Metoda vrac cestu k adresi se ablonami}
function GetTemplateFolder : string;
begin
   Result := RelToAbsPath(TEMPLATE_FOLDER,GetRunningScriptDir);
end;

{Metoda vrac cestu k ablon}
function GetTemplatePointName : string;
begin
   Result := RelToAbsFilename(TEMPLATE_POINT_NAME,GetTemplateFolder);
end;

{Metoda vrac cestu k ablon}
function GetTemplatePointDescription : string;
begin
   Result := RelToAbsFilename(TEMPLATE_POINT_DESCRIPTION,GetTemplateFolder);
end;

{Metoda vrac cestu k ablon}
function GetTemplateWaypointName : string;
begin
   Result := RelToAbsFilename(TEMPLATE_WAYPOINT_NAME,GetTemplateFolder);
end;

{Metoda vrac cestu k ablon}
function GetTemplateWaypointDescription : string;
begin
   Result := RelToAbsFilename(TEMPLATE_WAYPOINT_DESCRIPTION,GetTemplateFolder);
end;

function LogStatHasIssues(geo : TGeo) : boolean;
begin
   Geoget_DB.AddParamText(':id',geo.ID);
   Result := (Geoget_DB.GetTableValue('SELECT count(key) FROM geolog WHERE id = :id AND dt >= (SELECT max(dt) FROM geolog WHERE id = :id AND type IN (''Found it'',''Webcam Photo Taken'',''Attended'')) AND type NOT IN (''Found it'',''Webcam Photo Taken'',''Attended'') LIMIT 1;') > 0);
end;

{Pokud jsou po poslednim "Found it" logu i logy jineho typu, metoda vypise jejich statistiku}
function LogStat(geo: TGeo) : String;
var
   tab :TSQliteTable;
begin
   Result := '';
   if LogStatHasIssues(geo) then
   begin
     try
      Geoget_DB.AddParamText(':id',geo.ID);
      tab := Geoget_DB.GetTable('SELECT type, count(key) AS cnt FROM geolog WHERE id = :id AND dt >= (SELECT max(dt) FROM geolog WHERE id = :id AND type IN (''Found it'',''Webcam Photo Taken'',''Attended'')) AND type NOT IN (''Found it'',''Webcam Photo Taken'',''Attended'') GROUP BY type ORDER BY cnt DESC;', false);
      try
        while not tab.EOF do
        begin
          Result := Result + tab.FieldAsString(tab.FieldIndex['cnt']) + 'x ' + tab.FieldAsString(tab.FieldIndex['type']);
          if tab.next then Result := Result + ', ';
        end;
      finally
        tab.free;
      end;
     finally
     end;
   end;
end;

{Metoda zjistuje, zda je v listingu obsazeno nejake slovo ke zvyrazneni}
function ContainsHighlightWord(LongDesc: String) : boolean;
begin
    Result := RegexFind('(?si)\S*('+ AnsiToUtf(TEXT_HIGHLIGHT) +')\S*',LongDesc);
end;

{Metoda v listingu zvyrazni vyskyty definovanych klicovych slov}
function HighlightWords(LongDesc: String) : String;
begin
	if TEXT_HIGHLIGHT <> '' 
    then Result := RegexReplace('(?si)\S*('+ AnsiToUtf(TEXT_HIGHLIGHT) +')\S*', LongDesc, '<font color=RED><b>$0</b></font>', true)
    else Result := LongDesc;
end;

{Metoda vymaze text s additional waypointy pridavany groundspeakem na konec listingu v PQ}
function DeleteAddWptsInListing(LongDesc : String) : String;
begin
   if (CLEANUP_GROUNDSPEAK_WAYPOINTLIST = '1')
     then Result := RegexReplace('(?is)(<p>)?Additional Waypoints(</p>|<br>).*$', LongDesc, '', false)
     else Result := LongDesc;
end;

function ReplaceImages(Text : String) : String;
var img : String;
begin
   Result := Text;
   if (REPLACE_IMAGES = '1') Then
   begin
     img := AnsiToUtf(_('Image'));
     Result := RegexReplace('(?si)<\s*img[^>]*alt\s*=\s*["|'']([^>"'']*)["|''][^>]*>',Result,'<b>&lt;' + img + ': $1&gt;</b>',true); //vyseparovat z obrazku ALT
     //Result := RegexReplace('(?si)<\s*img[^>]*src\s*=\s*["|''][^>"'']*?([^>"''/]*)["|''][^>]*>',Result,'(' + img + ': $1)',true); //vyseparovat z neoALTovanych obrazku posledni cast SRC
     Result := RegexReplace('(?si)<\s*img[^>]*>',Result,'<b>&lt;' + img +'&gt;</b>',false); //pouze misto IMG zobrazime do zavorky upozorneni
   end;
end;

{Metoda ocisti predany text o nektere HTML tagy}
function CleanupHtml(Text : String) : String;
begin
    Result := Text;
    If (CLEANUP_HTML = '1') Then
    begin
      // Comment -> ''
      Result := RegexReplace('(?si)<!--.*?-->', Result, '', false);
      
      // TAG -> ''
      //SPAN, TABLE, TD, TR, TH, DIV, FONT, A, I, IMG, MARQUEE, CENTER, BLOCKQUOTE
      Result := RegexReplace('(?si)< *?(/|)?(span|table|td|tr|th|div|font|a|i|img|marquee|center|blockquote).*?>', Result, '', false);
    
      // TAG -> Paragraph
      // P, LI, UL, OL, H1 - H6
      Result := RegexReplace('(?si)< *?(/)?(p|li|ul|ol|h\d).*?>', Result, '<$1p>', true);

      //Remove unnecessary CRLF
      Result := ReplaceString(Result,CRLF, ' ');
    end;
end;

{Metoda vraci minimum z dvou predanych hodnot}
function Minimum(a, b : Integer) : Integer;
begin
  If a < b then Result := a
  else Result := b;
end;

{Metoda vrati nekolik poslednich logu na kesi naformatovanych do jednoducheho HTML}
function Logs(geo : TGeo) : String;
var n : Integer; s : String;
begin
  Result := '';
  if geo.Logs.count > 0 then 
  begin
    for n := 0 to Minimum(geo.Logs.count - 1,StrToInt(EXPORT_LOGS_MAXCOUNT)-1) do
    begin
      Result := Result + '<b>'+geo.Logs[n].Finder + ' (' + formatdatetime('dd". "mm". "yyyy', geo.Logs[n].Date) + ', ' + geo.Logs[n].LogType + '):</b><br>';
      s := RegexReplace('\n',geo.Logs[n].Text,'<br>',false);
      if CLEANUP_HTML = '1' then s := CleanupHtml(s);
      Result := Result + s + '<br><br>';
    end;
  end;
end;

{Metoda vrati informace o waypointech na kesi naformatovane do jednoducheho HTML}
function Waypoints(geo : Tgeo) : String;
var n : Integer;
begin
   Result := '';
   if (geo.Waypoints.Count > 0) then
   begin
      for n := 0 to geo.Waypoints.Count - 1 do
      begin
        if (n > 0) then Result := Result + '<br>';
        Result := 
        Result + '<b>' + geo.Waypoints[n].PrefixID + ' ' +
                 geo.Waypoints[n].Name + '</b> (' +
                 geo.Waypoints[n].WptType +')<br>'+ '  ' + 
                 ReplaceString(geo.Waypoints[n].getCoord, '', ' ') + '<br>';
        if geo.Waypoints[n].Description <> '' then Result := Result + geo.Waypoints[n].Description + '<br>';
        if geo.Waypoints[n].Comment <>     '' then Result := Result + ReplaceString(geo.Waypoints[n].Comment,CRLF, '<br>') + '<br>';
      end;
   end;
end;


function VarSubst_PoiGarminReplaceGeoBefore(geo:TGeo;sTemplate:string) : String;
begin
  BeforeCalled := true;
  Result := sTemplate;
  if (geo <> nil) then
  begin
    CleanLongDesc := '';
    CleanShortDesc := '';
    WmVisitInstructions := '';

    if(pos('%WMVISITINSTRUCTIONS%',Result)>0) then begin
      WmVisitInstructions := RegexSubstitute('(?si)<\s*(strong|b)\s*>Visit\s*Instructions:<\s*/\s*(strong|b)\s*>(<\s*br\s*/?\s*>)(.*)$',geo.LongDescription,'$4');
      CleanLongDesc:=RegexReplace('(?si)<\s*(strong|b)\s*>Visit\s*Instructions:<\s*/\s*(strong|b)\s*>(<\s*br\s*/?\s*>)(.*)$',geo.LongDescription,'',false);
    end;
    if((pos('%CONTAINSHIGHLIGHTWORD%',Result)>0) or (pos('%GARMIN_LONGLISTING%',Result)>0) or (pos('%GARMIN_LISTING%',Result)>0) or (pos('%GARMIN_SHORTLISTING%',Result)>0)) then 
    begin
      if (CleanLongDesc = '') 
        then CleanLongDesc := CleanupHtml(ReplaceImages(DeleteAddWptsInListing(geo.LongDescription)))
        else CleanLongDesc := CleanupHtml(ReplaceImages(DeleteAddWptsInListing(CleanLongDesc)));
      CleanShortDesc := CleanupHtml(ReplaceImages(geo.ShortDescription));
    end;
   
    if(pos('%LOGSTATHASISSUES%',Result)>0) then begin
      if LogStatHasIssues(geo) then Result:=ReplaceString(Result,'%LOGSTATHASISSUES%','true')
      else                          Result:=ReplaceString(Result,'%LOGSTATHASISSUES%','false');
    end;
    if(pos('%CONTAINSHIGHLIGHTWORD%',Result)>0) then begin
      if ContainsHighlightWord(CleanShortDesc + CRLF + CleanLongDesc) 
                               then Result:=ReplaceString(Result,'%CONTAINSHIGHLIGHTWORD%','true')
      else                          Result:=ReplaceString(Result,'%CONTAINSHIGHLIGHTWORD%','false');
    end;
  end;
end;

function VarSubst_PoiGarminReplaceGeoAfter(geo:TGeo;sTemplate:string) : String;
begin
  if not BeforeCalled then 
  begin
    ShowMessage(_('Error! Problem occured in registration and calling of custom functions at VarSubst library. This is critical failure!'));
    GeoAbort;
  end;
  BeforeCalled := false;
  Result := sTemplate;
  if (geo <> nil) then
  begin
    if(pos('%LOGSTAT%',Result)>0) then Result:=ReplaceString(Result,'%LOGSTAT%',VarSubstEncode(LogStat(geo)));
    if(pos('%LOGS%',Result)>0) then Result:=ReplaceString(Result,'%LOGS%',VarSubstEncode(Logs(geo)));
    if(pos('%WAYPOINTS%',Result)>0) then Result:=ReplaceString(Result,'%WAYPOINTS%',VarSubstEncode(Waypoints(geo)));  
    if(pos('%WMVISITINSTRUCTIONS%',Result)>0) then Result:=ReplaceString(Result,'%WMVISITINSTRUCTIONS%',VarSubstEncode(WmVisitInstructions));
    if(pos('%GARMIN_SHORTLISTING%',Result)>0) then Result:=ReplaceString(Result,'%GARMIN_SHORTLISTING%',VarSubstEncode(HighlightWords(CleanShortDesc)));
    if(pos('%GARMIN_LONGLISTING%',Result)>0) then Result:=ReplaceString(Result,'%GARMIN_LONGLISTING%',VarSubstEncode(HighlightWords(CleanLongDesc)));
    if(pos('%GARMIN_LISTING%',Result)>0) then Result:=ReplaceString(Result,'%GARMIN_LISTING%',VarSubstEncode(HighlightWords(CleanShortDesc + CRLF + CleanLongDesc)));    
  end;
end;

function VarSubst_PoiGarminReplaceWptBefore(wpt:TWpt;sTemplate:string) : String;
begin
  Result := sTemplate;
  if (wpt <> nil) 
    then Result := VarSubst_PoiGarminReplaceGeoBefore(wpt.parentgeo,Result);
end;

function VarSubst_PoiGarminReplaceWptAfter(wpt:TWpt;sTemplate:string) : String;
begin
  Result := sTemplate;
  if (wpt <> nil) 
    then Result := VarSubst_PoiGarminReplaceGeoAfter(wpt.parentgeo,Result);
end;

function VarSubst_PoiGarminGetDocumentationListBefore : TStrings;
begin
  Result := TStringList.Create;
  Result.Add(_('%CONTAINSHIGHLIGHTWORD% - true, if the listing contains any of the highlight words'));
  Result.Add(_('%LOGSTATHASISSUES% - true, if there are suspicious geocache logs after the last Found it log'));
end;

function VarSubst_PoiGarminGetDocumentationListAfter : TStrings;
begin
  Result := TStringList.Create;
  Result.Add(_('%GARMIN_LISTING% - whole listing processed by POI Garmin (HTML cleanup, word highlights, removed GS waypoint list in PQ)'));
  Result.Add(_('%LOGS% - prints the list of logs on a cache'));
  Result.Add(_('%LOGSTAT% - prints the list of suspicious geocache logs after the last Found it log'));
  Result.Add(_('%GARMIN_LONGLISTING% - ShortDescription part of listing processed by POI Garmin (HTML cleanup, word highlights, removed GS waypoint list in PQ)'));
  Result.Add(_('%GARMIN_SHORTLISTING% - ShortDescription part of listing processed by POI Garmin (HTML cleanup, word highlights)'));  
  Result.Add(_('%WAYPOINTS% - prints the list of additional waypoints on a cache'));
  Result.Add(_('%WMVISITINSTRUCTIONS% - waymark visit instructions. If you use this pattern, visit instructions will be automatically removed from the end of the listing to avoid duplicates'));  
end;

procedure VarSubst_PoiGarminRegister;
var listBefore,listAfter : TStrings;
begin
   CleanShortDesc := '';
   CleanLongDesc := '';
   WmVisitInstructions := '';
   BeforeCalled := false;
   
   listBefore := VarSubst_PoiGarminGetDocumentationListBefore;
   listAfter := VarSubst_PoiGarminGetDocumentationListAfter;
   try
      VarSubstAddFunctionReplaceGeo(@VarSubst_PoiGarminReplaceGeoBefore,listBefore,true);
      VarSubstAddFunctionReplaceWpt(@VarSubst_PoiGarminReplaceWptBefore,listBefore,true);
      VarSubstAddFunctionReplaceGeo(@VarSubst_PoiGarminReplaceGeoAfter,listAfter,false);
      VarSubstAddFunctionReplaceWpt(@VarSubst_PoiGarminReplaceWptAfter,listAfter,false);
   finally
      listBefore.Free;
      listAfter.Free;
   end;
end;
